import { EquipmentSlot } from "@minecraft/server";
import { ARMOR_PROTECTION_VALUES } from "./armors.constantes";
import { distanceVector } from "./utils/distance-between-vector3";
export class DamageHandler {
    static getDamageTakenWithProtection(armorProtection, ArmorToughness, damage, enchantLevel) {
        const top = Math.max(armorProtection / 5, armorProtection - ((4 * damage * (1 / enchantLevel)) / ArmorToughness + 8));
        return 1 - top / 25;
    }
    static handleArmorDurability(entity, damage) {
        const equipment = entity.getComponent('equippable');
        const damageInHeart = damage * 2;
        const durabilityDamage = damageInHeart / 4;
        for (const equipmentSlot in EquipmentSlot) {
            if (equipmentSlot === 'Mainhand' || 'Offhand')
                continue;
            const item = equipment.getEquipment(EquipmentSlot[equipmentSlot]);
            if (!item)
                continue;
            const itemDurability = item.getComponent('durability');
            itemDurability.damage -= durabilityDamage;
            equipment.setEquipment(EquipmentSlot[equipmentSlot], item);
        }
    }
    static getKnockbackResistance(hit) {
        const equipment = hit.getComponent('equippable');
        if (!equipment)
            return 1;
        const headEquipment = equipment.getEquipment(EquipmentSlot.Head);
        const chestEquipment = equipment.getEquipment(EquipmentSlot.Chest);
        const legsEquipment = equipment.getEquipment(EquipmentSlot.Legs);
        const feetEquipment = equipment.getEquipment(EquipmentSlot.Feet);
        const helmetKnockbackResistance = ARMOR_PROTECTION_VALUES[headEquipment?.typeId]?.knockback_resistance || 0;
        const chestplateKnockbackResistance = ARMOR_PROTECTION_VALUES[chestEquipment?.typeId]?.knockback_resistance || 0;
        const leggingsKnockbackResistance = ARMOR_PROTECTION_VALUES[legsEquipment?.typeId]?.knockback_resistance || 0;
        const feetKnockbackResistance = ARMOR_PROTECTION_VALUES[feetEquipment?.typeId]?.knockback_resistance || 0;
        const total = helmetKnockbackResistance + chestplateKnockbackResistance + leggingsKnockbackResistance + feetKnockbackResistance;
        return 1 - (total * 0.07);
    }
    static handleArmorMultiplier(hit, damage) {
        const equipment = hit.getComponent('equippable');
        if (!equipment)
            return 1;
        const headEquipment = equipment.getEquipment(EquipmentSlot.Head);
        const chestEquipment = equipment.getEquipment(EquipmentSlot.Chest);
        const legsEquipment = equipment.getEquipment(EquipmentSlot.Legs);
        const feetEquipment = equipment.getEquipment(EquipmentSlot.Feet);
        const helmetProtection = ARMOR_PROTECTION_VALUES[headEquipment?.typeId]?.protection || 0;
        const chestplateProtection = ARMOR_PROTECTION_VALUES[chestEquipment?.typeId]?.protection || 0;
        const leggingsProtection = ARMOR_PROTECTION_VALUES[legsEquipment?.typeId]?.protection || 0;
        const feetProtection = ARMOR_PROTECTION_VALUES[feetEquipment?.typeId]?.protection || 0;
        const maxProtection = helmetProtection + chestplateProtection + leggingsProtection + feetProtection;
        const helmetToughness = ARMOR_PROTECTION_VALUES[headEquipment?.typeId]?.toughness || 0;
        const chestplateToughness = ARMOR_PROTECTION_VALUES[chestEquipment?.typeId]?.toughness || 0;
        const leggingsToughness = ARMOR_PROTECTION_VALUES[legsEquipment?.typeId]?.toughness || 0;
        const feetToughness = ARMOR_PROTECTION_VALUES[feetEquipment?.typeId]?.toughness || 0;
        const maxToughness = helmetToughness + chestplateToughness + leggingsToughness + feetToughness;
        const helmetProtectionEnchant = headEquipment?.getComponent('enchantable')?.getEnchantment('protection')?.level || 0;
        const chestplateProhelmetProtectionEnchant = chestEquipment?.getComponent('enchantable')?.getEnchantment('protection')?.level || 0;
        const leggingsProhelmetProtectionEnchant = legsEquipment?.getComponent('enchantable')?.getEnchantment('protection')?.level || 0;
        const feetProhelmetProtectionEnchant = feetEquipment?.getComponent('enchantable')?.getEnchantment('protection')?.level || 0;
        const helmetProProtectionEnchant = headEquipment?.getComponent('enchantable')?.getEnchantment('protection')?.level || 0;
        const chestplateProjectilProhelmetProtectionEnchant = chestEquipment?.getComponent('enchantable')?.getEnchantment('protection')?.level || 0;
        const leggingsProjectilProhelmetProtectionEnchant = legsEquipment?.getComponent('enchantable')?.getEnchantment('protection')?.level || 0;
        const feetProjewssWithEnchant = helmetToughness + chestplateToughness + leggingsToughness + feetToughness;
        const totalProtection = feetProjewssWithEnchant + helmetProProtectionEnchant + chestplateProjectilProhelmetProtectionEnchant + leggingsProjectilProhelmetProtectionEnchant + helmetProtectionEnchant + chestplateProhelmetProtectionEnchant + leggingsProhelmetProtectionEnchant + feetProhelmetProtectionEnchant;
        return DamageHandler.getDamageTakenWithProtection(maxProtection, maxToughness, damage, totalProtection) || 1;
    }
    static getDamages(weapon, hit, bullet) {
        const damages = weapon.damage - (weapon.damageDropOff * Math.floor(distanceVector(bullet.spawnLocation, hit.location)));
        if (damages <= 0)
            return 0;
        const armor = this.handleArmorMultiplier(hit, damages);
        if (armor == 1)
            return (damages);
        return (damages * ((this.handleArmorMultiplier(hit, damages) * (weapon.armorPenetration))));
    }
}
