import MeleeWeapon from 'melee/melee-weapon';
export default class Runebreaker extends MeleeWeapon {
    constructor() {
        super(...arguments);
        this.itemTypeId = 'melee:runebreaker';
        // MAIN MELEE PROPERTIES
        this.name = 'Runebreaker';
        this.drawTime = 1050; // in ms
        this.displayDrawCooldown = true;
        this.leftAttack = {
            enabled: true,
            damages: 60,
            delay: 2,
            cooldown: 20,
            displayCooldown: true,
            range: 6,
            precision: 0.07,
            sideAttack: {
                timeToSlash: 150, // in ms
                rayPerLine: 15,
                borderRadius: 0, // in percent
                lineCount: 1,
                areaHeight: 2.3, // in blocks
                areaWeight: {
                    from: -30, // in degree
                    to: 30, // in degree
                },
            },
            sounds: {
                onHitBlock: {
                    soundIds: ['cso.combatknife.wall'],
                    pitch: 1,
                    volume: 1.5,
                },
                onHitEntity: {
                    soundIds: ['cso.tomahawk.hit'],
                    pitch: 1,
                    volume: 1.5,
                },
                onMiss: {
                    soundIds: [''],
                    pitch: 0.5,
                    volume: 100,
                },
            },
            particles: {
                onHitBlock: {
                    particleIds: ['runebreaker:wall'],
                },
                onHitEntity: {
                    particleIds: ['ag:blood','ag:blood','ag:blood','ag:blood'],
                },
            },
        };
        this.rightAttack = {
            enabled: true,
            damages: -1,
            delay: 20,
            fire: {
                time : 1000, // in ms,
                useEffects : true
            },
            cooldown: 0,
            range: 0.3,
            displayCooldown: true,
            precision: 0.07,
            sideAttack: {
                timeToSlash: 150, // in ms
                rayPerLine: 15,
                borderRadius: 0, // in percent
                lineCount: 1,
                areaHeight: 1.3, // in blocks
                areaWeight: {
                    from: -20, // in degree
                    to: 20, // in degree
                },
            },
            sounds: {
                onHitBlock: {
                    soundIds: [''],
                    pitch: 1,
                    volume: 1.5,
                },
                onHitEntity: {
                    soundIds: [''],
                    pitch: 1,
                    volume: 1.5,
                },
                onMiss: {
                    soundIds: [''],
                    pitch: 0.5,
                    volume: 100,
                },
            },
            particles: {
                onHitBlock: {
                    particleIds: ['minecraft:pois5'],
                },
                onHitEntity: {
                    particleIds: ['minecraft:pois5'],
                },
            },
        };
        // ATTACK on draw
        this.initialAttack = {
            enabled: false,
        };
        // Charge System
        this.charges = {
            enabled: true,
            fire: {
                time : 1000, // in ms,
                useEffects : true
            },
            timePerSquare: [570, 750, 700, 510],
            functionsTriggeredPerSquare: ['runebreaker_3', 'runebreaker_3', 'runebreaker_3', 'runebreaker_3'],
            functionsTriggeredAtReleasedPerSquare: ['runebreaker_0', 'runebreaker_0', 'runebreaker_0', 'runebreaker_none'],
            functionsTriggeredAtReleasedAfterDelayPerSquare: ['runebreaker_none', 'runebreaker_none', 'runebreaker_none', 'runebreaker_1'],
            damagePerSquare: [30, 60, 90, 120],
            displaySquares: true,
            triggerAttackIfLastSquareIsReached: false
        };
        this.chargeAttack = {
            enabled: true,
            damages: -1,
            delay: 10,
            cooldown: 0,
            range: 6,
            fire: {
                time : 1000, // in ms,
                useEffects : true
            },
            knockback: {
                Ximpusle: 3,
                Zimpusle: 0.6,
            },
            precision: 0.07,
            sideAttack: {
                timeToSlash: 150, // in ms
                rayPerLine: 25,
                borderRadius: 50, // in percent
                lineCount: 25,
                areaHeight: 4.3, // in blocks
                areaWeight: {
                    from: -25, // in degree
                    to: 25, // in degree
                },
            },
            sounds: {
                onHitBlock: {
                    soundIds: ['cso.combatknife.wall'],
                    pitch: 1,
                    volume: 3,
                },
                onHitEntity: {
                    soundIds: ['cso.tomahawk.hit'],
                    pitch: 1,
                    volume: 3,
                },
            },
            particles: {
                onHitBlock: {
                    particleIds: ['runebreaker:wall'],
                },
                onHitEntity: {
                    particleIds: ['ag:blood','ag:blood','ag:blood','ag:blood','ag:blood','ag:blood','ag:blood','ag:blood','ag:blood'],
                },
            },
        };
        this.sounds = {
            onDraw: {
                soundIds: [''],
                pitch: 1,
                volume: 0.2,
            },
        };
        this.particles = {
            onDraw: {
                particleIds: [''],
            },
        };
    }
}
