import { system } from '@minecraft/server';
export class ChargeLoadingBar {
    constructor(weapon, player) {
        this.weapon = weapon;
        this.player = player;
        this.oldColorIndex = null;
        this.currentColorIndex = null;
        this.CONFIG = {
            chargeColor: '§a',
            defaultColor: '§7',
            waitingColor: '§8',
            squareSymbol: '■',
        };
        player.startedHoldingRightClickIntervalId = system.runInterval(() => {
            if (!this.weapon.charges?.enabled)
                return;
            if (!this.weapon) {
                player.resetRightClickHolding();
                if (this.weapon.charges.displaySquares)
                    player.onScreenDisplay.setActionBar(`${' '.repeat(this.weapon.charges.timePerSquare.length).split('').join(' ')}`);
                return;
            }
            if (!player.ableToHitDate.isFinished) {
                if (this.weapon.charges.displaySquares)
                    player.onScreenDisplay.setActionBar(`${this.CONFIG.waitingColor}${this.CONFIG.squareSymbol.repeat(this.weapon.charges.timePerSquare.length).split('').join(' ')}`);
                player.startedHoldingRightClickDate = Date.now();
                return;
            }
            this.update();
            player.chargeIndex = this.currentColorIndex === 0 ? this.weapon.charges.timePerSquare.length : this.currentColorIndex;
            if (player.chargeIndex === this.weapon.charges.timePerSquare.length && this.weapon.charges.triggerAttackIfLastSquareIsReached) {
                this.weapon.onReleaseHoldingRightClick(player, false);
                player.chargeIndex = 0;
                this.oldColorIndex = 0;
                this.currentColorIndex = 0;
                player.startedHoldingRightClickDate = Date.now();
            }
            player.runCommandAsync(`/camerashake add @s ${player.chargeIndex * 0.003} 0.3 rotational`);
            if (this.weapon.charges.displaySquares)
                player.onScreenDisplay.setActionBar(this.textToRender);
        }, 1);
    }
    update() {
        if (!this.weapon.charges?.enabled)
            return;
        const baseArray = Array(this.weapon.charges.timePerSquare.length - 1).fill(`${this.CONFIG.squareSymbol} `);
        baseArray.push(this.CONFIG.squareSymbol);
        const timePassedFromTheBegining = Date.now() - this.player.startedHoldingRightClickDate;
        for (let i = 0; i < this.weapon.charges.timePerSquare.length + 1; i++) {
            const squareTime = i <= 0 ? this.weapon.charges.timePerSquare[0] : this.weapon.charges.timePerSquare.slice(0, i).reduce((a, b) => a + b);
            if (timePassedFromTheBegining - this.weapon.charges.timePerSquare.reduce((a, b) => a + b) >= 0) {
                this.currentColorIndex = this.weapon.charges.timePerSquare.length;
                break;
            }
            if (timePassedFromTheBegining - squareTime <= 0) {
                this.currentColorIndex = i - 1;
                break;
            }
        }
        if (this.oldColorIndex !== this.currentColorIndex && this.weapon.charges.functionsTriggeredPerSquare?.length > 0) {
            this.oldColorIndex = this.currentColorIndex;
            const functionName = this.weapon.charges.functionsTriggeredPerSquare[this.oldColorIndex - 1];
            if (functionName)
                this.player.runCommandAsync(`function ${functionName}`);
        }
        baseArray.splice(this.currentColorIndex <= 0 ? baseArray.length : this.currentColorIndex, 0, this.currentColorIndex <= 0 ? this.CONFIG.chargeColor : this.CONFIG.defaultColor);
        this.textToRender = this.CONFIG.chargeColor + (this.currentColorIndex <= 0 ? this.CONFIG.defaultColor : '') + baseArray.join('');
    }
}
